"""Low level client for the Syncthing REST API."""

import asyncio

import aiohttp
from yarl import URL

from .exceptions import SyncthingError


class API:
    """Low level client."""

    DEFAULT_TIMEOUT = 10

    def __init__(
        self,
        api_key,
        url="http://127.0.0.1:8384",
        timeout=DEFAULT_TIMEOUT,
        verify_ssl=True,
        loop=None,
        session=None,
    ):
        """Initialize the client."""
        self._api_key = api_key
        self._url = URL(url)
        self._timeout = aiohttp.ClientTimeout(total=timeout)
        self._verify_ssl = verify_ssl

        self._loop = loop or asyncio.get_event_loop()
        self._session = session
        self._close_session = False

        if self._session is None:
            self._session = aiohttp.ClientSession(loop=self._loop)
            self._close_session = True

    async def request(self, *args, **kwargs):
        """Perform request with error wrapping."""
        try:
            return await self.raw_request(*args, **kwargs)
        except Exception as error:
            raise SyncthingError from error

    async def raw_request(self, uri, params=None, data=None, method="GET"):
        """Perform request."""
        async with self._session.request(
            method,
            self._url.join(URL(uri)).update_query(params),
            json=data,
            headers={"Accept": "application/json", "X-API-Key": self._api_key,},
            timeout=self._timeout,
            verify_ssl=self._verify_ssl,
        ) as response:
            response.raise_for_status()
            return await response.json()

    async def close(self):
        """Close the session."""
        if self._session and self._close_session:
            await self._session.close()



"""Entrypoint for the database Syncthing REST API."""


class Database:
    """Entrypoint class for the database Syncthing REST API."""

    def __init__(self, api):
        """Initialize the client."""
        self._api = api

    async def status(self, folder_id):
        """Get folder status."""
        return await self._api.request("/rest/db/status", params={"folder": folder_id})



"""Entrypoint for the events Syncthing REST API."""

import asyncio

from .exceptions import SyncthingError


class Events:
    """Entrypoint class for the events Syncthing REST API."""

    def __init__(self, api):
        """Initialize the client."""
        self._last_seen_id = 0
        self._api = api

    @property
    def last_seen_id(self):
        """Get last seen event's id."""
        return self._last_seen_id

    async def listen(self):
        """Listen to events."""
        self._last_seen_id = 0

        while True:
            try:
                events = await self._api.raw_request(
                    "/rest/events", params={"since": self._last_seen_id}
                )
                for event in events:
                    yield event
                self._last_seen_id = events[-1]["id"]
            except asyncio.TimeoutError:
                continue
            except asyncio.CancelledError:
                return
            except Exception as error:
                raise SyncthingError from error



"""Entrypoint for the system Syncthing REST API."""

from .exceptions import PingError


class System:
    """Entrypoint class for the system Syncthing REST API."""

    def __init__(self, api):
        """Initialize the client."""
        self._api = api

    async def ping(self):
        """Check server availability."""
        try:
            result = await self._api.raw_request("/rest/system/ping")
            if (
                not isinstance(result, dict)
                or "ping" not in result
                or result["ping"] != "pong"
            ):
                raise PingError
        except Exception as error:
            raise PingError from error

    async def config(self):
        """Get server config."""
        return await self._api.request("/rest/system/config")

    async def status(self):
        """Get server config."""
        return await self._api.request("/rest/system/status")



"""Exceptions for the Syncthing REST API."""


class SyncthingError(Exception):
    """Base Syncthing Exception class."""


class PingError(SyncthingError):
    """When server does not respond to ping."""
